//-------------------------------------------------------------------//
//                                                                   //
//  JFC Copyright (C) Glenn Rosenthal, 1999,2000,2001.               //
//  All rights reserved.                                             //
//                                                                   //
//-------------------------------------------------------------------//

//-------------------------------------------------------------------
//
//  This modlue handles the install properties of JFC.
//
//  Noramlly every time JFC runs, it checks to see if the file 
//  extensions are associated with JFC.  If any of the exntensions
//  are not associated then it attempts to do an install.  The 
//  install consists of assigning the extensions to JFC, and 
//  possibly putting JFC in the start-menu and/or the desktop.
//  The program also checks to make sure the file associations point 
//  back to this executable.
//

#include "jfc_main.h"
#include "jfc_conf.h"
#include "jfc_help.h"
#include "jfc_inst.h"
#include "jfc_wnce.h"
#include "jfc_misc.h"
#include <shellapi.h>
#include <shlobj.h>

//
//  This routine saves a shell link.  A shell link is the name for 
//  a link object, in this case it will be an item in the start menu,
//  or an item on the desktop.
//
//      file  -- System Shell interface file structure already intiailized.
//      place -- Key indicating what system location to place the object.
//               The keys of interest are CSIDL_PROGRAMS (program directory 
//               of the start-menu) and CSIDL_DESKTOP (desktop).
//      dir   -- Subdirectory of the indicated directory to place the
//               object.
//

//
//  Windows CE version of the create link.
//
#ifdef WINCE

static void save_link (TCHAR *target,int place,TCHAR *dir) {
  TCHAR buffer[SIZE_BUFFER];

  get_folder (place,buffer);                                // Get base dir.
  if (dir && *dir) {                                        // Add sub-directory
    lstrcat (buffer,TEXT("\\"));
    lstrcat (buffer,dir);
    CreateDirectory (buffer,NULL);
  }
  lstrcat (buffer,TEXT("\\JFC.lnk"));                       // This needs to have the extension .lnk to make an actuall link on CE.
  DeleteFile       (buffer);
  SHCreateShortcut (buffer,target);
  return;
}

//
//  Windows 95/NT version of creating a link.
//
#else WINCE
static void save_link (IPersistFile *file,int place,TCHAR *dir) {
  TCHAR buffer[SIZE_BUFFER];

  get_folder (place,buffer);                                // Get base dir.
  if (dir && *dir) {                                        // Add sub-directory
    lstrcat (buffer,TEXT("\\"));
    lstrcat (buffer,dir);
    CreateDirectory (buffer,NULL);
  }
  lstrcat (buffer,TEXT("\\"));                              // Add name.
  lstrcat (buffer,TEXT("JFC.lnk"));
#ifdef UNICODE
  file->Save (buffer,true);
#else
  WORD wbuffer[SIZE_BUFFER];
  MultiByteToWideChar (CP_ACP,0,buffer,-1,wbuffer,SIZE_BUFFER);
  file->Save (wbuffer,true);
#endif UNICODE
  return;
}
#endif WINCE

//-------------------------------------------------------------------
//
//  Static definitions and data.
//

                                // First install dialog box return codes.
#define INSTALL_ABORT       0   // Abort the install
#define INSTALL_ADVANCED    1   // Go to the advanced install
#define INSTALL_OK          2   // Causes a simple automatic install.

//
//  Structure used to pass information to the advanced install dialog box.
//
struct adv_install {
  int   ext;                // Indicates .JFC should be associated with this program.                             
  int   start;              // If non-zero causes the program to be placed in the Start Menu.
  int   desktop;            // If non-zero causes the program to be placed on the desktop.
  TCHAR group[SIZE_BUFFER]; // Group to place program in in the start menu.
};

static struct adv_install *adv_install; // Instance.

//-------------------------------------------------------------------
//
//  Dialog box procedures.
//

//
//  Dialog box function for the advnaced install dialog box.
//
//  On entrance, the adv_isntall structure is filled out.  The ext
//  array indicates which extensions buttons are active (disabled 
//  buttons have already been assigned to JFC).  On exit the ext
//  field indicates which items should be assigned to JFC.
//
//      IDC_AISTART     Start-menu check box.
//      IDC_AIGROUP     Start-menu group.
//      IDC_AIDESKTOP   Desktop checkbox.
//      IDC_AIJCE       First extension (the others are in series).
//
static BOOL CALLBACK dialog_advinstall (HWND hwnd,UINT message,WPARAM wParam,LPARAM lParam) {
  switch (message) {
    case WM_INITDIALOG:
         CheckDlgButton (hwnd,IDC_AIJFC    ,true);
         CheckDlgButton (hwnd,IDC_AISTART  ,adv_install->start);
         CheckDlgButton (hwnd,IDC_AIDESKTOP,adv_install->desktop);
         SetDlgItemText (hwnd,IDC_AIGROUP  ,adv_install->group);
         EnableWindow   (GetDlgItem(hwnd,IDC_AIJFC),adv_install->ext);
         return (true);
    case WM_HELP:
         do_help (hwnd,IDH_INSTALL_ADVANCED);
         return  (true);
    case WM_COMMAND:
         switch (LOWORD(wParam)) {
           INPUT_CHECK (IDC_AIGROUP);
           case IDC_AISTART:
                EnableWindow (GetDlgItem(hwnd,IDC_AIGROUP),IsDlgButtonChecked(hwnd,IDC_AISTART));
                return (true);
           case IDCANCEL:
                EndDialog (hwnd,true);
                return    (true);
           case IDOK:
                adv_install->ext     = IsDlgButtonChecked(hwnd,IDC_AIJFC);
                adv_install->start   = IsDlgButtonChecked(hwnd,IDC_AISTART);
                adv_install->desktop = IsDlgButtonChecked(hwnd,IDC_AIDESKTOP);
                GetDlgItemText (hwnd,IDC_AIGROUP,adv_install->group,SIZE_BUFFER);
                EndDialog (hwnd,false);
                return    (true);
         }
         break;
  }
  return (false);
}

//
//  Dialog box handler for the first install dialog box.  This dialog box 
//  allows the user to select advanced install, automatic install,
//  abort the install, and/or supress further checking for installs.
//
//      ID_IDADVANCED   Advanced install button.
//      ID_IDINSTALL    Check-box to supress further checking.
//
static BOOL CALLBACK dialog_install (HWND hwnd,UINT message,WPARAM wParam,LPARAM lParam) {
  switch (message) {
    case WM_INITDIALOG:
         CheckDlgButton (hwnd,IDC_IDINSTALL,!jfc_config.cfg.install);
         return (true);
    case WM_HELP:
         do_help (hwnd,IDH_INSTALL_MORE);
         return  (true);
    case WM_COMMAND:
         jfc_config.cfg.install = !IsDlgButtonChecked(hwnd,IDC_IDINSTALL);
         switch (LOWORD(wParam)) {
           case IDC_IDADVANCED:
                EndDialog (hwnd,INSTALL_ADVANCED);
                return    (true);
           case IDCANCEL:
                EndDialog (hwnd,INSTALL_ABORT);
                return    (true);
           case IDOK:
                EndDialog (hwnd,INSTALL_OK);
                return    (true);
         }
         break;
  }
  return (false);
}

//-------------------------------------------------------------------
//
//  Static routines.
//

//
//  This routine gets the value of a key from the registry.
//
//      subkey -- Subkey to get.  All keys retreived here are from the 
//                HKEY_CLASSES_ROOT class.
//      buffer -- Buffer to hold the result.  This should be SIZE_BUFFER
//                in length.
//
//      RETURN -- A non-zero return value indicates an error.
//
static int get_key (TCHAR *subkey,TCHAR *buffer) {
  HKEY    hkey;
  ulong   size,type;
  size = SIZE_BUFFER;
  if (ERROR_SUCCESS != RegOpenKeyEx(HKEY_CLASSES_ROOT,subkey,0,KEY_QUERY_VALUE,&hkey)) return (true);
  RegQueryValueEx (hkey,NULL,NULL,&type,(byte *) buffer,&size);
  RegCloseKey     (hkey);
  return (false);
}

//
//  Sets the value in a registery key.  All keys set by this routine are
//  in the HKEY_CLASSES_ROOT class.
//
//      subkey -- Key location.
//      format -- Format for key value, followed by sprintf parameters.
//
#define ZSTRSIZE(x) (sizeof(TCHAR)*(lstrlen(x)+1))

static void make_key (TCHAR *subkey,TCHAR *format,...) {
  HKEY    hkey;
  TCHAR   value[SIZE_BUFFER];
  ulong   type;
  va_list argptr;
  va_start  (argptr,format);
  wvsprintf (value ,format,argptr);
  RegCreateKeyEx (HKEY_CLASSES_ROOT,subkey,NULL,NULL,0,KEY_ALL_ACCESS,NULL,&hkey,&type);
  RegSetValueEx  (hkey,NULL,NULL,REG_SZ,(byte *) value,ZSTRSIZE(value));
  RegCloseKey    (hkey);
  return;
}

//-------------------------------------------------------------------
//
//  Exported routines.
//

//
//  Checks to see if JFC is installed, and if it is not allows the 
//  user to install it.
//
//      force -- If set, will force JFC to allow an install.  Can be 
//               used to generate start-menu/desktop items when JFC 
//               cannot tell if the item already exists.
//
#define REG_COMMAND TEXT("JFC\\Shell\\Open\\Command")

void do_install (int force) {
  static TCHAR* file_ext = TEXT(".jfc");
  int    ok;
  TCHAR  buffer[SIZE_BUFFER],command[SIZE_BUFFER],executable[SIZE_BUFFER];
  struct adv_install install;

  if (!force && !jfc_config.cfg.install) return;    // User does not want check.
  ok = !force;
//
//  Initialize values.
//
  adv_install     = &install;
  install.start   = true;
  install.desktop = false;
  GET_STRING (install.group,IDS_INST_GROUP);
  if (!GetModuleFileName(instance,executable,SIZE_BUFFER)) return;
  sprintf (command,TEXT("\"%s\""),executable);
//
//  Check extension.
//
  if (get_key(file_ext,buffer)) { install.ext = true; ok = false; }
  else if (!stricmp(TEXT("JFC"),buffer)) install.ext = false;
  else { install.ext = true; ok = false; }
//
//  Check executable command.
//
  if (get_key(REG_COMMAND,buffer)) ok = false;
  else if (strnicmp(command,buffer,lstrlen(command))) ok = false;
//
//  Do we need to do an install.
//  
  if (ok) return;
//
//  Do the dialog boxes.
//
  switch (DialogBox(language,MAKEINTRESOURCE(IDD_INSTALL),main_window,(DLGPROC) dialog_install)) {
    case INSTALL_ABORT:
         return;
    case INSTALL_ADVANCED:
         if (DialogBox(language,MAKEINTRESOURCE(IDD_ADVINSTALL),main_window,(DLGPROC) dialog_advinstall)) return;
         break;
    case INSTALL_OK:
         break;
  }
//
//  Install the extensions and the file association.
//
  if (install.ext) make_key (file_ext,TEXT("JFC"));
  make_key (TEXT("JFC"             ),get_string(IDS_INST_FILETYPE));
  make_key (TEXT("JFC\\DefaultIcon"),TEXT("%s,-%d"),executable,IDI_FILEICON);
  make_key (REG_COMMAND             ,TEXT("%s \"%%1\""),command);
//
//  Check for start-menu and/or desktop options.
//
  if (install.start || install.desktop) {
//
//  Create links for Windows CE.
//
#ifdef WINCE
    if (install.start  ) save_link (command,CSIDL_PROGRAMS,install.group);
    if (install.desktop) save_link (command,CSIDL_DESKTOP ,NULL         );
//
//  Create links for Windows 95/NT
//
#else WINCE
    IShellLink   *link; 
    IPersistFile *file; 
    if (S_OK == CoCreateInstance(CLSID_ShellLink,NULL,CLSCTX_INPROC_SERVER,IID_IShellLink,(void **) &link)) {
      link->SetPath        (executable);      // This is what the shortcut points to.
      link->SetDescription ("JFC");           // This is what will show up in the start-menu.
      if (S_OK == link->QueryInterface(IID_IPersistFile,(void **) &file)) {     // Create file object.
        if (install.start  ) save_link (file,CSIDL_PROGRAMS,install.group);
        if (install.desktop) save_link (file,CSIDL_DESKTOP ,NULL         );
        file->Release ();                     // Done with file.
      }
    }
    link->Release ();                         // Done with link.
#endif WINCE
  }
  return;
}





