//-------------------------------------------------------------------//
//                                                                   //
//  JFC Copyright (C) Glenn Rosenthal, 1999,2000,2001.               //
//  All rights reserved.                                             //
//                                                                   //
//-------------------------------------------------------------------//

//-------------------------------------------------------------------
//
//  This modlue is a collection of micelaeous routines not placed 
//  in any other module.
//
#include "jfc_main.h"
#include "jfc_help.h"
#include "jfc_misc.h"
#include <commctrl.h>   // Needed for tab controls
#include <shellapi.h>
#include <shlobj.h>

//-------------------------------------------------------------------
//
//  Static rotuines.
//

//
//  Main dialog box handler for tabed-dialog boxes.  There is actually a second user
//  handler tha may be called to process messages not processed here.
//
static BOOL CALLBACK tab_dialog (HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) {
  HWND      page,tab;           // Window pointer for page and tab control.
  TabSetup *setup;              // Pointer to setup structure.
  TC_ITEM   item;               // Item structure to access data in the tab control.
  RECT      dlg,rect;           // Rectangles for the dialog box and the tab control (and it's window)
  int       i;              
  tab   = GetDlgItem(hwnd,IDC_TABCONTROL);              // The tab control.
  setup = (TabSetup *) GetWindowLong(hwnd,DWL_USER);    // The user setup, passed in lParam, and saved in the DWL_USER window param
  switch (msg) {
    case WM_INITDIALOG:
         setup = (TabSetup *) lParam;                   // Get the real setup.
         SetWindowLong (hwnd,DWL_USER,lParam);          // Save it for later
         GetWindowRect (hwnd,&dlg);                     // Get working space in tab control
         dlg.top    += GetSystemMetrics(SM_CYCAPTION)+GetSystemMetrics(SM_CYDLGFRAME);
         dlg.bottom += GetSystemMetrics(SM_CYCAPTION)+GetSystemMetrics(SM_CYDLGFRAME);
//       dlg.left   += GetSystemMetrics(SM_CXDLGFRAME);
//       dlg.right  += GetSystemMetrics(SM_CXDLGFRAME);
         memset (&item,0,sizeof(item));
         item.mask = TCIF_TEXT | TCIF_PARAM;
         for (i = 0; i < setup->count; i++) {           // Create each page.
           page = JCreateDialog(setup->pages[i].id,hwnd,setup->pages[i].procedure);
           item.pszText = get_string(setup->pages[i].text);     // Add page to tab control
           item.lParam  = (LPARAM) page;
           TabCtrl_InsertItem (tab,i,&item);
           GetWindowRect      (tab,&rect);              // Center page in tab control window.
           TabCtrl_AdjustRect (tab,false,&rect);
           MoveWindow         (page,rect.left-dlg.left,rect.top-dlg.top,rect.right-rect.left,rect.bottom-rect.top,true);
           ShowWindow         (page,SW_HIDE);        
         }
         TabCtrl_SetCurSel (tab,setup->page);           // Select last page used.
         goto SetPage;
    case WM_HELP:
         do_help (hwnd,setup->pages[setup->page].help);
         return  (true);
    case WM_NOTIFY:
         switch (((LPNMHDR) lParam)->code) {
           case TCN_SELCHANGING:                        // Page is going out of view so hide it.
                item.mask = TCIF_PARAM;
                TabCtrl_GetItem(tab,TabCtrl_GetCurSel(tab),&item);
                ShowWindow ((HWND) (item.lParam),SW_HIDE);
                return (false);
           case TCN_SELCHANGE:                          // Page is coming into view so show it.
SetPage:;
                item.mask   = TCIF_PARAM;
                i           = TabCtrl_GetCurSel(tab);
                setup->page = i;
                TabCtrl_GetItem (tab,i,&item);
                ShowWindow (page = ((HWND) (item.lParam)),SW_SHOW);
                BringWindowToTop (page);
                if (msg == WM_INITDIALOG) SetFocus (GetNextDlgTabItem(page,NULL,false));
                return (false);                
         }
         break;
    case WM_COMMAND:
         switch (LOWORD(wParam)) { 
           case IDOK:                                   // Keep the changes, so read all pages.
                item.mask = TCIF_PARAM;
                for (i = 0; i < setup->count; i++) {
                  TabCtrl_GetItem (tab,i,&item);
                  SendMessage ((HWND) (item.lParam),WM_GETDLGVALUES,0,0);
                }
           case IDCANCEL:                               // Just exit.
                EndDialog (hwnd,LOWORD(wParam == IDOK));
                return    (true);
           case IDC_TABHELP:
                do_help (hwnd,setup->pages[setup->page].help);
                return  (true);
         }
         break;
  }
//
//  Call user procedure.
//
  if (setup && setup->procedure) return ((*setup->procedure)(hwnd,msg,wParam,lParam));
  return (false);
}

//-------------------------------------------------------------------
//
//  Group of routines to process button dialog boxes.  A button 
//  dialog box is any dialog box that contains only active buttons.
//  Additionally, a single static text item is supported.  The retun
//  value from this type of dialog is the ID value of the button 
//  selected.  Any button will terminate the dialog.
//
//      IDC_DATA    // ID for static text item.
//
static TCHAR *button_data;  // Static location to pass data to the 
                            //   dialog box procedure for the static 
                            //   text item.  A value of NULL will 
                            //   suppress the text.
static int    button_help;  // Static location used to hold help ID for
                            //   button dialog.

//
//  Dialog box procedure.
//
static BOOL CALLBACK dialog_button (HWND hwnd,UINT message,WPARAM wParam,LPARAM lParam) {
  switch (message) {
    case WM_INITDIALOG: 
         if (button_data) SetDlgItemText (hwnd,IDC_DATA,button_data);
         return (true);
    case WM_HELP:
         do_help (hwnd,button_help);
         return  (true);
    case WM_COMMAND:
         EndDialog (hwnd,LOWORD(wParam));
         return    (true);
  }
  return (false);
}

//
//  Generates a button dialog.
//
//      idd    -- Diolog box id.
//      data   -- Data for the static text item.  A value of NULL should
//                be used if there is not static text item.
//      help   -- Help id for dialog
//
//      RETURN -- The ID of the button selected that termianted the 
//                dialog.
//
int ButtonDialog (int idd,TCHAR *data,int help) {
  button_data = data;
  button_help = help;
  return (JDialogBox(idd,main_window,(DLGPROC) dialog_button));
}

//-------------------------------------------------------------------
//
//  Exported rotuines.
//

//
//  Generate an error message.
//
//      error  -- Set non-zero for an error and zero for a warning.
//      format -- printf style format string id.
//      ...    -- Arguments for printf string.
//
void ErrorMessage (int error,int format,...) {
  TCHAR buffer[SIZE_BUFFER],string[SIZE_BUFFER];
  va_list argptr;
  va_start   (argptr,format);
  GET_STRING (string,format);
  wvsprintf  (buffer,string,argptr);
  if (error) MessageBox (main_window,buffer,get_string(IDS_ERROR),MB_OK | MB_ICONWARNING);
    else MessageBox (main_window,buffer,get_string(IDS_WARNING),MB_OK | MB_ICONINFORMATION);
  return;
}

//
//  This routine formats a string based on an ID from the resource table.
//
//      buffer -- Buffer to hold the formatted string.
//      id     -- ID of printf tyep format pattern.
//      
//      RETURN -- Pointer to buffer.
//
TCHAR *format_string (TCHAR *buffer,int id,...) {
  TCHAR format[SIZE_BUFFER];
  va_list argptr;
  va_start  (argptr,id);
  LoadString (language,id,format,SIZE_BUFFER);
  wvsprintf (buffer,format,argptr);
  return (buffer);
}

//
//  Get a system folder.
//
//      id -- Folder ID.
//      buffer -- Buffer for folder path.
//
//      RETURN -- Pointer to buffer.
//
TCHAR *get_folder (int id,TCHAR *buffer) {
  LPITEMIDLIST item  = NULL;
  LPMALLOC     alloc = NULL;
  SHGetMalloc                (&alloc);
  SHGetSpecialFolderLocation (NULL,CSIDL_PERSONAL,&item);
  SHGetPathFromIDList        (item,buffer); 
  alloc->Free    (item);
  alloc->Release ();
  return (buffer);
}

//
//  Gets an interger value from a dialog box edit control with 
//  bounds checking, and a default value to use if no value can 
//  be extracted.
//
//      hwnd    -- Dialog box window pointer.
//      id      -- ID of edit control.
//      min_val -- Minimum acceptable value.
//      max_val -- Maximum acceptable value.
//      def     -- Default value (used if user clears the control, etc.).
//
//      RETURN  -- Return value is the inteteger.
//
int get_int (HWND hwnd,int id,int min_val,int max_val,int def) {
  BOOL err;
  int  i;
  i = GetDlgItemInt (hwnd,id,&err,true);
  if (!err) return (def);
  if (i < min_val) i = min_val;
  if (i > max_val) i = max_val;
  return (i);
}

//
//  Gets a string from the system resource and returns the string.
//
//  Care must be used in calling this routine since the routine returns the string in 
//  a static data space.  First this means there are two rules to follow:
//
//      1. The length of the string that can be recovered is limited by the the
//         static buffer.  Tis currently SIZE_BUFFER.  The filter strings are long!
//      2. There is only one static buffer so care must be taken in the use of 
//         the string.
//
//      id     -- ID of string to recover.
//
//      RETURN -- Pointer to static location containning the string.
//
TCHAR *get_string (int id) {
  static TCHAR buffer[SIZE_BUFFER];
  LoadString (language,id,buffer,SIZE_BUFFER);
  return (buffer);
}

//
//  This is a replacment for the system routines CreateDialogParam/CreateDialog.  This 
//  routien supports reading the templete from one source and the dialog from another.
//  This is necessary to allow custom controls defined in JWPce to be used in dialog 
//  boxes generated from templets in a DLL.  
//
//      id     -- ID for dialog box.
//      hwnd   -- Parent window.
//      proc   -- Dialog procedure.
//      param  -- Parameter (optional), passed to dialog procedure.
//
//      RETURN -- Pointer to dialog window generated.
//
HWND JCreateDialog (int id,HWND hwnd,DLGPROC proc,long param) {
  HRSRC   handle;
  HGLOBAL resource; 
  void   *data;
  handle   = FindResource(language,MAKEINTRESOURCE(id),RT_DIALOG);
  resource = LoadResource(language,handle);
  data     = LockResource(resource);
  return (CreateDialogIndirectParam(instance,(LPCDLGTEMPLATE) data,hwnd,proc,param));
}

//
//  This is a replacment for the system routines DialogBoxParam/DialogBox.  This 
//  routien supports reading the templete from one source and the dialog from another.
//  This is necessary to allow custom controls defined in JWPce to be used in dialog 
//  boxes generated from templets in a DLL.  
//
//      id     -- ID for dialog box.
//      hwnd   -- Parent window.
//      proc   -- Dialog procedure.
//      param  -- Parameter (optional), passed to dialog procedure.
//
//      RETURN -- Pointer to dialog window generated.
//
int JDialogBox (int id,HWND hwnd,DLGPROC proc,long param) {
  HRSRC   handle;
  HGLOBAL resource; 
  void   *data;
  handle   = FindResource(language,MAKEINTRESOURCE(id),RT_DIALOG);
  resource = LoadResource(language,handle);
  data     = LockResource(resource);
  return (DialogBoxIndirectParam(instance,(LPCDLGTEMPLATE) data,hwnd,proc,param));
}

//
//  This is an enhanced version of the system routine.
//
//      hwnd    -- Parent window for dialog box.
//      text    -- ID for text used in the message box.  This is indicates a printf style 
//                 format string.
//      caption -- ID for message box title.
//      type    -- Type for message box, all system types are supported.
//      ...     -- Parameters for use with the text string.
//
//      RETURN  -- MessageBox() return value.
//
int JMessageBox (HWND hwnd,int text,int caption,UINT type,...) {
  TCHAR buffer[SIZE_BUFFER],message[SIZE_BUFFER],title[SIZE_BUFFER];
  va_list argptr;
  va_start   (argptr,type);
  GET_STRING (message,text);
  wvsprintf  (buffer,message,argptr);
  GET_STRING (title,caption);
  return (MessageBox(hwnd,buffer,title,type));
}

//
//  Duplicate a kanji string.
//
//      string -- String to be duplicated (can be NULL).
//
//      RETURN -- Pointer to memory allocated string.
//
KANJI *kdupstr (KANJI *string) {
  int    i;
  KANJI *ptr;
  if (!(i = kstrlen(string))) return (NULL);
  i = sizeof(KANJI)*(i+1);
  if (ptr = (KANJI *) malloc(i)) memcpy (ptr,string,i);
  return (ptr);
}

//
//  Copy a string from the source to the destination
//
//      dest   -- Destiation buffer
//      source -- Source buffer
//
void kstrcpy (KANJI *dest,KANJI *source) {
  while (*source) *dest++ = *source++;
  *dest = 0;
  return;
}

//
//  Determine the length of a kanji string.
//
//      string -- String to find the length of (can be NULL).
//
//      RETURN -- Length of string.
//
int kstrlen (KANJI *string) {
  int i;
  if (!string) return (0);
  for (i = 0; string[i]; i++);
  return (i);
}

//
//  Check to see if a file exists.
//
//      name   -- Name of file to check.
//
//      RETURN -- A non-zero return value indicates that the file 
//                exists and can be open.
//
int FileExists (TCHAR *name) {
  HANDLE file;
#ifndef WINCE
  UINT   error_mode;
  error_mode = SetErrorMode(SEM_FAILCRITICALERRORS);    // Diable errors incase we are using a floppy
#endif
  file = CreateFile(name,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
#ifndef WINCE
  SetErrorMode (error_mode);                            // Re-enable error handling.
#endif
  if (file == INVALID_HANDLE_VALUE) return (false);
  CloseHandle (file);
  return (true);
}

//
//  This routine gets information about a specific menu item.  The 
//  information returned includes the ID and the text of the item.
//
//      menu     -- Menu or sub-menu to look in.
//      item     -- Menu item or position (from zero).
//      position -- If this is non-zero then item indicates a psotion,
//                  otherwise this idicates a menu item by id.
//      buffer   -- Location to return the text of the menu item.  
//                  This buffer should be atleast SIZE_BUFFER long to
//                  avoid an error.
//
//      RETURN   -- Return value is the ID of the menu item, or zero
//                  if this is not a valid menu item.
//
//  This routine replaced the old GetMenuItemText, and GetMenuItemID,
//  which are not supported under windows CE.
//  
long get_menudata (HMENU menu,int item,int position,TCHAR *buffer) {
  MENUITEMINFO info;
  info.cbSize     = sizeof(info);           // This is required, but not in the docs.
  info.fMask      = MIIM_ID | MIIM_TYPE;    // What we want back.
  info.dwTypeData = buffer;                 // Buffer setup
  info.cch        = SIZE_BUFFER;            // Size of buffer, also required but not in the docs.
  if (!GetMenuItemInfo(menu,item,position,&info)) return (0);
  return (info.wID);
}

//
//  Generate an out of memory error.
//
void OutOfMemory () {
  ErrorMessage (true,IDS_ERROR_MEMORY);
  return;
}

//
//  Random number generator.  This generates a random number between 0 and max-1.
//
//      max    -- Determines the maximum random number that can be generated.
//
//      RETURN -- Random number between 0 and max-1.
//
int random (int max) {
  int i,j;
  if (max <= 1) return (0);
  for (i = 1; i < max; i = i << 1);
  i--;
  do {
    j = i & rand();
  } while (j >= max);
  return (j);
}

//
//  Small stub rotuine for initializing a tabed dialog box.
//
//      ID     -- ID for the main window.
//      setup  -- A fully initialized setup structure, that describes each page and it's 
//                dialog box handler.
//
//      RETURN -- Unless the client main routine returns something, a true value indicates 
//                the user wants the values, and a false indicates he/she aborted the 
//                dialog box.
//
int TabDialog (int id,TabSetup *setup) {
  return (JDialogBox(id,main_window,(DLGPROC) tab_dialog,(LONG) setup));
}

//
//  Generate a simple Yes-No dialog box, and get input from the user.
//
//      format -- printf style format string passed to the dialog box.
//
//      RETURN -- A non-zero return value indicates yes.
//
int YesNo (int format,...) {
  TCHAR buffer[SIZE_BUFFER],string[SIZE_BUFFER];
  va_list argptr;
  va_start   (argptr,format);
  GET_STRING (string,format);
  wvsprintf  (buffer,string,argptr);
  return (IDYES == MessageBox(main_window,buffer,get_string(IDS_AREYOUSURE),MB_YESNO | MB_ICONQUESTION));
}

//
//  Gets a string from the resources and convert tabs to zeros.
//
//      buffer -- Buffer to store string in.
//      id     -- ID of string to bet.
//
//      RETURN -- Pointer to the buffer
//
TCHAR *zero_string (TCHAR *buffer,int id) {
  TCHAR *ptr;
  LOAD_STRING (buffer,id,SIZE_BUFFER);
  for (ptr = buffer; *ptr; ptr++) {
    if (*ptr == '\t') *ptr = 0;
  }
  return (buffer);
}
