//-------------------------------------------------------------------//
//                                                                   //
//  JFC Copyright (C) Glenn Rosenthal, 1999,2000,2001.               //
//  All rights reserved.                                             //
//                                                                   //
//-------------------------------------------------------------------//

//-------------------------------------------------------------------
//
//  This modlue manages the status bar.  
//

#include "jfc_main.h"
#include "jfc_conf.h"
#include "jfc_file.h"
#include "jfc_font.h"
#include "jfc_stat.h"
#include "jfc_misc.h"

//-------------------------------------------------------------------
//
//  class JFC_stat.
//
//  This class manages the status bar.
//

class JFC_stat jfc_stat;                        // Class instance.

static LRESULT CALLBACK JFC_stat_proc (HWND hwnd,UINT iMsg,WPARAM wParam,LPARAM lParam);

//
//  Window procedure.
//
static LRESULT CALLBACK JFC_stat_proc (HWND hwnd,UINT iMsg,WPARAM wParam,LPARAM lParam) {
  HDC         hdc;
  PAINTSTRUCT ps;

  switch (iMsg) {
    case WM_PAINT:
         hdc = BeginPaint (hwnd,&ps);
         jfc_stat.draw(hdc);
         EndPaint (hwnd,&ps);
         return (0);
  }
  return (DefWindowProc(hwnd,iMsg,wParam,lParam));
}

//
//  Constructor
//
JFC_stat::JFC_stat () {
  height = 0;
  window = NULL;
  return;
}

//
//  Adjust the size of the window (is a responce to a size change).
//
void JFC_stat::adjust () {
  RECT rect;
  HDC  hdc;
  SIZE size;
  if (!window) return;
  height  = (3*sysfont_height)/2;               // Setup window size and move it.
  GetClientRect (main_window,&rect);
  MoveWindow    (window,-1,rect.bottom-height+1,rect.right+2,height,true);
//
//  Setup internal size parameters in the window.
//
  offset   = (1*sysfont_height)/5;              // Vertical offset for text.
  hdc      = GetDC (window);
  GetTextExtentPoint32 (hdc,TEXT("I"), 1,&size);
  x_text   = (short) size.cx;                   // Location for text message
  ReleaseDC (window,hdc);
  return;
}

//
//  Render the status bar.  This routine is not called directly, but
//  rather is called from the window procedure.
//
//      hdc -- Display context.
//
void JFC_stat::draw (HDC hdc) {
  TCHAR buffer[SIZE_BUFFER];
  SetBkMode (hdc,TRANSPARENT);
  if (!jfc_file || !jfc_file->flash) format_string(buffer,IDS_NOCARDS);
    else format_string(buffer,IDS_STATUS,jfc_file->flash_count,jfc_file->right,jfc_file->wrong);
  TextOut (hdc,x_text,offset,buffer,lstrlen(buffer));
  return;
}

//
//  Intialize the class and setup.
//
//      wclass -- class tructure passed in with intialization parameters.
//                if this is NULL we do not need to register class.
//
int JFC_stat::initialize (WNDCLASS *wclass) {
  if (wclass) {                                 // Register class.
    wclass->style         = CS_HREDRAW | CS_VREDRAW;
    wclass->lpfnWndProc   = JFC_stat_proc;
    wclass->hbrBackground = (HBRUSH) (COLOR_MENU+1);
    wclass->lpszClassName = TEXT("JFC-Stat");
    if (!RegisterClass(wclass)) return (true);
  }
  if (!jfc_config.cfg.status && window) {       // This is actually invoided it the user
    DestroyWindow (window);                     //   changes the configuration to delete
  }                                             //   an existing status bar.
  if (!jfc_config.cfg.status) {                 // No status bar.
    window  = NULL;
    height  = 0;
    return (false);
  }
  if (window) return (true);                    // Alread have a window.
  if (!(window = CreateWindow(TEXT("JFC-Stat"),NULL,WS_CHILD | WS_VISIBLE | WS_BORDER,0,0,0,0,main_window,(HMENU) 3,instance,NULL))) return (true);
  adjust ();
  return (false);
}

//
//  Redraw the screen.  This simply checks to see if there is a window
//  then ivalidates the window.
//
void JFC_stat::redraw () {
  if (window) InvalidateRect(window,NULL,true); 
  return;
}

//
//  End class JFC_stat
//
//-------------------------------------------------------------------



